type var = string

type t = Const of Q.t
       | Var of var
       | Plus of t * t
       | Times of t * t

let zero = Const Q.zero

module Syntax = struct
  let (!) c = Const (Q.of_float c)

  let v x =
    if String.get x 0 = '_' then
      failwith (x ^ " is an invalid variable")
    else Var x

  let (+) x y = Plus(x, y)

  let ( * ) x y = Times(x, y)

  let (-) x = Const(Q.of_int (-1)) * x
end

let rec occurs_in x = function
| Var y       -> x = y
| Plus(l, r)
| Times(l, r) -> occurs_in x l || occurs_in x r
| _           -> false

let rec subst x e = function
| Var y when x = y -> e
| Plus(a, b)       -> Plus(subst x e a, subst x e b)
| Times(a, b)      -> Times(subst x e a, subst x e b)
| t                -> t
