open Core

type t = Multi.t list

open Multi

let pzero n = List.init n ~f:(Fn.const zero)

let norm p q =
  let n = List.length p in
  let m = List.length q in
  let (p, q, n, m) = if n < m then (p, q, n, m) else (q, p, m, n) in
  p @ pzero (m - n), q

open Multi.Syntax

let add p q = let (p, q) = norm p q in List.map2_exn p q ~f:(+)

let scale c = List.map ~f:(( * ) c)

let multiply p =
  List.foldi ~init:[] ~f:(fun i s pi -> add s (pzero i @ scale pi p))

let integrate c p =
  c :: List.mapi p ~f:(fun i p_i -> p_i * Const (Q.div Q.one (Q.of_int (succ i))))

let rec of_term m = function
| Var x as v -> begin match List.Assoc.find m x ~equal:(=) with
  | Some p -> p
  | None   -> [v]
end
| Plus(a, b)  -> add (of_term m a) (of_term m b)
| Times(a, b) -> multiply (of_term m a) (of_term m b)
| c           -> [c]

let rec (^) x i = if i = 0 then Const Q.one else Var x * (x ^ (pred i))

let to_term x = List.foldi ~init:zero ~f:(fun i p' pi -> p' + pi * (x ^ i))
